/***************************************************************************************************
*                       Sample for the usage of TopoART and Hypersphere TopoART                    *
*                           Created by Marko Tscherepanow, 30 December 2012                        *
***************************************************************************************************/

// Compile: mcs /r:LibTopoART.dll TopoART_sample2.cs
// Run: mono TopoART_sample2.exe

using System;
using System.IO;
using System.Text.RegularExpressions;
using LibTopoART;
using System.Globalization;

namespace LibTopoART_samples
{
	/// <summary>
	/// Sample using artificial two-dimensional data.
	/// <para>
	/// Train TopoART or Hypersphere TopoART with a two-dimensional dataset similar to the one used in "Marko 
	/// Tscherepanow and Sören Riechers, 'An Incremental On-line Classifier for Imbalanced, Incomplete, and 
	/// Noisy Data', European Conference on Artificial Intelligence (ECAI), Workshop on Active and Incremental 
	/// Learning (AIL), pp. 18-23, 2012." This dataset comprises six clusters (each containing 15,000 samples) 
	/// as well as 10,000 noise samples. These samples were mixed randomly.
	/// </para>
	/// <para>The resulting neural network can be visualised using the R script <c>ShowTopoARTResults.R</c> or the 
	/// R script <c>ShowHypersphereTopoARTResults.R</c>, respectively. Both R scripts are provided in the 
	/// subfolder <c>R</c>.
	/// </para>
	/// </summary>
	class TopoART_sample2
	{
		enum Selector {
			USE_TopoART,
			USE_Fast_TopoART,
			USE_Hypersphere_TopoART
		};

	    static void Main() {
	
			// Dataset
			string Dataset 			=	"data/AIL12-like_dataset.txt";		
			long SampleNumber		=	100000;
			long InputDimension		=	2;

			ITopoART t				=	null;
			string FilePrefix		=	null;
			decimal[][] InputArray	=	new decimal[SampleNumber][];
			StreamReader DatasetFile;
			long i;
			string Line;
			DateTime start,end;
			TimeSpan time;

			// Choose TopoART implementation (see enum Selector above)
			Selector sel = Selector.USE_TopoART;

			// Start time measuring
			start = DateTime.Now;
			
			// Load dataset
			using(DatasetFile = new StreamReader(File.Open(Dataset, FileMode.Open))) {
				for(i = 0; i < SampleNumber; ++i) {
					Line = DatasetFile.ReadLine();
					string[] numbers = Regex.Split(Line, @"\s+");
					InputArray[i] = new decimal[InputDimension]; 
					InputArray[i][0] = Decimal.Parse(numbers[1], NumberStyles.Float,System.Globalization.CultureInfo.InvariantCulture);
					InputArray[i][1] = Decimal.Parse(numbers[2] ,NumberStyles.Float,System.Globalization.CultureInfo.InvariantCulture);
				}
			}
			
			// Initialise a TopoART network with appropriate parameter values
			switch(sel)
			{
				case Selector.USE_TopoART:
					t = new TopoART(InputDimension, 2, 0.92m);	
					t.Beta_sbm	=	0.65m;
					t.Phi		=	5;
					t.Tau		=	200;
					FilePrefix 	=	"TopoART";
					break;
				case Selector.USE_Fast_TopoART:
					t = new Fast_TopoART(InputDimension, 2, 0.92m);	
					t.Beta_sbm	=	0.65m;
					t.Phi		=	5;
					t.Tau		=	200;
					FilePrefix 	=	"Fast_TopoART";
					break;
				case Selector.USE_Hypersphere_TopoART:
					t = new Hypersphere_TopoART(InputDimension, 2, 0.90m);
					t.Phi		=	5;
					t.Tau		=	200;
					t.Beta_sbm	=	0.60m;
					FilePrefix 	=	"Hypersphere_TopoART";
					break;
			}
					
			// Train network
			for(i = 0; i < SampleNumber; ++i) {
				t.Learn(InputArray[i]);
				if(i % 1000 == 0) Console.Write(".");
			}
			Console.WriteLine("");
			
			// Determine clusters
			t.ComputeClusterIDs();
			
			// Save network in human readable form
			t.SaveText(FilePrefix + "_AIL12-like_dataset.txt");

			// Stop time measuring
			end = DateTime.Now;

			// Output the required time
			time = end - start;
			Console.WriteLine("time for computation: " + time);
		}
	}
}